@@ -1,4 +1,8 @@
 # -*- coding: utf-8 -*-
+#
+# This file is part of pyfesom2
+# Original code by Dmitry Sidorenko, Qiang Wang, Sergey Danilov and Patrick Scholz
+#
 
 """Top-level package for pyfesom2."""
 
@@ -7,9 +11,9 @@ __email__ = "koldunovn@gmail.com"
 __version__ = "0.1.0"
 
 from .load_mesh_data import *
-from .fesom_plot_tools import *
+from .plotting import *
 from .climatology import *
-from .regriding import *
+from .regridding import *
 from .transect import *
 from .diagnostics import *
 from .ut import *
